/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.service;

import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.entity.MaterialInformation;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.entity.OnPassageMaterials;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.mapper.OnPassageMaterialsMapper;
import com.jeeplus.modules.sg.balancedlibrary.reportPerson.entity.ReportPerson;
import com.jeeplus.modules.sg.balancedlibrary.reportPerson.mapper.ReportPersonMapper;
import org.apache.shiro.crypto.hash.Hash;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;


/**
 * 在途物资Service
 * @author 徐滕
 * @version 1.0
 */
@Service
@Transactional(readOnly = true)
public class OnPassageMaterialsService extends CrudService<OnPassageMaterialsMapper, OnPassageMaterials> {

    @Autowired
    private ReportPersonMapper reportPersonMapper;

    public void comparativeData(List<OnPassageMaterials> onPassageMaterialsList, List<MaterialInformation> materialInformationList){

        //计划编制人为空
        List<OnPassageMaterials> isEmptyList = new ArrayList<>();
        //统计物料编码 部门
        Map<String, BigDecimal> materialCountMap = new HashMap<>();
        //统计物料编码 计划编制人相同合同数量
        Map<String, BigDecimal> sameMap = new HashMap<>();
        //统计计划编制人为空合同数量
        Map<String, BigDecimal> isEmptyMap = new HashMap<>();
        //两表比较
        List<String> compareList = new ArrayList<>();
        /**************************************************************************/


        //从数据库中索取所有部门及提报人
        Map<String, List<ReportPerson>> reportPersonMap = getReportPerson();

        //循环比较找出 计划编制人为空添加isEmptyMap 根据计划编制人找出所在部门 物料编码-部门为去重条件
        onPassageMaterialsList.forEach(opm -> {
            //找到对应部门
            String reportPersonOfDepartment = getReportPersonOfDepartment(reportPersonMap,opm.getPlanner());
            //如果没有部门 表示空部门
            String key = opm.getMaterialCode() + "-" + reportPersonOfDepartment;
            if (sameMap.containsKey(key)) {
                sameMap.put(key, new BigDecimal(opm.getContractCount()).add(sameMap.get(key)));
            } else {
                sameMap.put(key, new BigDecimal(opm.getContractCount()));
            }
        });


        materialInformationList.forEach(m -> {
            String key = m.getMaterialCode() + "-" + m.getReportingDepartment();
            if (materialCountMap.containsKey(key)) {
					materialCountMap.put(key, new BigDecimal(m.getTotal()).add(materialCountMap.get(key)));
            } else {
                if(!m.getMaterialCode().isEmpty()&&!m.getTotal().isEmpty()){

                    materialCountMap.put(key, new BigDecimal(m.getTotal()));
                }
            }
        });

        materialCountMap.forEach((k,v)->{
            if(sameMap.containsKey(k)){
                if(sameMap.get(k).compareTo(v)>=0){
                    compareList.add(k);
                }
            }
        });

        materialInformationList.forEach(l->{
            String key = l.getMaterialCode() + "-" + l.getReportingDepartment();
            if(compareList.contains(key)){
                l.setRemarkText("在途已有" + sameMap.get(key)+"单位，请核实本次需求");
            }
        });

    }

    /**
     * 查询数据库获取对应的部门及部门下的人
     * @return
     */
    private Map<String,List<ReportPerson>> getReportPerson(){
        Map<String,List<ReportPerson>> map = new HashMap<>();
        List<ReportPerson> departmentGroupByPerson = reportPersonMapper.findDepartmentGroupByPerson();
        departmentGroupByPerson.forEach(l->{
            if(map.containsKey(l.getReportDepartment())){
                map.get(l.getReportDepartment()).add(l);
            }else{
                List<ReportPerson> reportPeople = new ArrayList<>();
                reportPeople.add(l);
                map.put(l.getReportDepartment(),reportPeople);
            }
        });
        return map;
    }

    /**
     * 获取提报人部门
     */
    private String getReportPersonOfDepartment(Map<String, List<ReportPerson>> reportPerson,String name){
        AtomicReference<String> departmentName = new AtomicReference<>();
        reportPerson.forEach((k,v)->{
            v.forEach(l->{
                if(l.getReportPerson().equals(name)){
                    departmentName.set(k);
                }
            });
        });
        return departmentName.get();
    }

}